unit uLCMeter;

interface

uses
  uCommon, uSignalIn, uSignalOut,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TeEngine, Series, ExtCtrls, TeeProcs, Chart, StdCtrls, ComCtrls;

const
  cWavesCount = 9; // Cycle from 0 to cWaveCount

type
  Rel = Extended; // Real value
  Com = record // Complex value
   case Byte of
     0: (x, y: Rel);
     1: (u: array [0..1] of Rel);
   end;

  TfmLCR = class(TForm)
    Chart1: TChart;
    Series1: TFastLineSeries;
    Series2: TFastLineSeries;
    PnlDeviceConnection: TPanel;
    shLED: TShape;
    Label2: TLabel;
    Label3: TLabel;
    lblZ: TLabel;
    Shape1: TShape;
    Shape29: TShape;
    Shape2: TShape;
    Label4: TLabel;
    Label5: TLabel;
    lblLeft: TLabel;
    Shape3: TShape;
    lblRight: TLabel;
    Shape4: TShape;
    shpR: TShape;
    shpRGnd: TShape;
    Label6: TLabel;
    Shape23: TShape;
    Shape27: TShape;
    Shape28: TShape;
    Shape8: TShape;
    Shape14: TShape;
    Shape9: TShape;
    ShCC: TShape;
    btnStart: TButton;
    cbIn: TComboBox;
    cbOut: TComboBox;
    rgRunMode: TRadioGroup;
    cbSwap: TCheckBox;
    edR: TEdit;
    Label7: TLabel;
    Label8: TLabel;
    cbGraphScale: TCheckBox;
    cbFixAxis: TCheckBox;
    shpRUp: TShape;
    DelayTimer: TTimer;
    reMes: TRichEdit;
    Label1: TLabel;
    cbOutSrc: TComboBox;
    cbInSrc: TComboBox;
    lblR: TLabel;
    Series3: TFastLineSeries;
    cbBits: TComboBox;
    Label9: TLabel;
    Label10: TLabel;
    cbSampling: TComboBox;
    Label11: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnStartMeasurementClick(Sender: TObject);
    procedure btnStartCalibrationClick(Sender: TObject);
    procedure OnStart(Sender: TObject);
    procedure OnStop(Sender: TObject);
    procedure OnStopOffSetCalibr(Sender: TObject);
    procedure OnStopAmplitudeCalibr(Sender: TObject);
    procedure btnStopMeasurementClick(Sender: TObject);
    procedure btnStopCalibrationClick(Sender: TObject);
    procedure cbInChange(Sender: TObject);
    procedure cbOutChange(Sender: TObject);
    procedure rgRunModeClick(Sender: TObject);
    procedure cbSwapClick(Sender: TObject);
    procedure edRChange(Sender: TObject);
    procedure Chart1ClickAxis(Sender: TCustomChart; Axis: TChartAxis;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure cbGraphScaleClick(Sender: TObject);
    procedure cbFixAxisClick(Sender: TObject);
    procedure OnDelayTimer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cbOutSrcChange(Sender: TObject);
    procedure cbInSrcChange(Sender: TObject);
    procedure cbBitsSamplingChange(Sender: TObject);
  private
    { Dclarations prives }
    FlblLLeft,
    FlblRLeft: Integer;
    FSignalOut: TSignalOut;
    FSignalIn: TSignalIn;
    FTime: TDateTime;
    FR: Extended;
    FCounter: Integer;
    FCanClose: Boolean;
    FBuffer: array[0..cWavesCount] of record
      KZ: Com;
      W: Rel;
    end;
    function DataFunction(const aData: TDataMatrix): Integer;
    function ShowFunction(const aData: TDataMatrix): Integer;
    function OnSaturationCalibration(aValue: Integer): Integer;
    function OnSaturationMeasurement(aValue: Integer): Integer;
    function OnLowSignalMeasurement(aValue: Integer): Integer;
    procedure OnShowMessage(aMes: String; aState: Boolean);
  public
    { Dclarations publiques }
  end;

var
  fmLCR: TfmLCR;

const
  Com0: Com = (x: 0.0; y: 0.0);

implementation

uses
  uFour1, IniFiles, uMixerLog;

{$R *.DFM}
const
  sAppSection = 'LC Meter';

var OldDecSep: Char;

// --- Com lib -----------------------------------------------------------------

function cCom (const a): com;
begin
  Result:= Com(a);
end;

function cMod2 (const a: Com): Rel;
begin
  Result:= sqr(a.x) + sqr(a.y);
end;

function cAdd (const a, b: Com): Com;
var i: Integer;
begin
  for i:= Low(a.u) to High(a.u) do
    Result.u[i]:= a.u[i] + b.u[i];
end;

function cSub (const a, b: Com): Com;
var i: Integer;
begin
  for i:= Low(a.u) to High(a.u) do
    Result.u[i]:= a.u[i] - b.u[i];
end;

function cInv (const a: Com): Com;
var t, d: Rel;
begin
  Result:= Com0;
  if (a.x = 0) and (a.y = 0) then
    Exit;
  if abs(a.x) >= abs(a.y) then begin
    t:= a.y / a.x;
    d:= a.x + t * a.y;
    Result.x:= 1 / d;
    Result.y:= - t / d;
  end else begin
    t:= a.x / a.y;
    d:= a.y + t * a.x;
    Result.x:=   t / d;
    Result.y:=  -1 / d;
  end;
end;

function cDiv(a, b: Com): Com;
var t, d: Rel;
begin
  Result:= Com0;
  if ((a.x = 0) and (a.y = 0)) or ((b.x = 0) and (b.y = 0)) then
    Exit;
  if abs(b.x) >= abs(b.y) then begin
    t:= b.y / b.x;
    d:= b.x + t * b.y;
    Result.x:= (a.x + a.y * t) / d;
    Result.y:= (a.y - a.x * t) / d;
  end else begin
    t:= b.x / b.y;
    d:= b.y + t * b.x;
    Result.x:= (a.x * t + a.y) / d;
    Result.y:= (a.y * t - a.x) / d;
  end;
end;

function cMuR (const a: Com; const b: Rel): Com;
var i: Integer;
begin
  for i:= Low(a.u) to High(a.u) do
    Result.u[i]:= a.u[i] * b;
end;

function cCon (const a: Com): Com;
begin
  Result.x:=  a.x;
  Result.y:= -a.y;
end;

// --- Float string  -----------------------------------------------------------

function ExtStr(aValue: Extended): String;
const cPrecision = 3;
      cDigits = 0;
begin
  if Abs(aValue) > 1E3 then
  if Abs(aValue) > 1E6 then
    Result:= FloatToStrF(aValue * 1E-6, ffGeneral, cPrecision, cDigits) + ' M'
  else
    Result:= FloatToStrF(aValue * 1E-3, ffGeneral, cPrecision, cDigits) + ' k'
  else
  if Abs(aValue) < 1.0 then
  if Abs(aValue) < 1E-3 then
  if Abs(aValue) < 1E-6 then
  if Abs(aValue) < 1E-9 then
    Result:= FloatToStrF(aValue * 1E12, ffGeneral, cPrecision, cDigits) + ' p'
  else
    Result:= FloatToStrF(aValue * 1E9,  ffGeneral, cPrecision, cDigits) + ' n'
  else
    Result:= FloatToStrF(aValue * 1E6,  ffGeneral, cPrecision, cDigits) + ' '
  else
    Result:= FloatToStrF(aValue * 1E3,  ffGeneral, cPrecision, cDigits) + ' m'
  else
    Result:= FloatToStrF(aValue,        ffGeneral, cPrecision, cDigits) + ' ';
end;

// --- Form itself -------------------------------------------------------------

procedure TfmLCR.FormCreate(Sender: TObject);
begin
  if RelVer then begin
    Caption:= 'LCR meter';
    DelayTimer.Interval:= 100;
  end else begin
    Caption:= 'LC meter free 2';
    lblZ.Top:= (lblZ.Top + lblR.Top) div 2;
    lblR.Visible:= False;
    DelayTimer.Interval:= 500;
  end;
  OldDecSep:= DecimalSeparator;
  DecimalSeparator:= '.';
  Chart1.Legend.Color:= clBtnFace;
  edRChange(Self);
  FlblLLeft:= lblLeft.Left;
  FlblRLeft:= lblRight.Left;
  rgRunModeClick(Self);
  Chart1.RightAxis.Grid.Visible:= False;
  reMes.Clear;
  reMes.ReadOnly:= True;
  FCanClose:= False;
end;

procedure TfmLCR.btnStartMeasurementClick(Sender: TObject);
var i: Integer;
begin
  FSignalIn.Stop;
  FSignalOut.Stop;
  OnShowMessage('Start measurement...', True);
  for i:= 0 to FSignalOut.Channels - 1 do
    FSignalOut.Attenuator[i]:= 0;
  if FSignalOut.Sampling > FSignalIn.Sampling then
    FSignalOut.Sampling:= FSignalIn.Sampling;
  if FSignalIn.Sampling > FSignalOut.Sampling then
    FSignalIn.Sampling:= FSignalOut.Sampling;
  FSignalOut.MultiSin(cWavesCount,1);
  FSignalIn.Count:= FSignalOut.Count;
  FSignalIn.OnSaturation:= OnSaturationMeasurement;
  FSignalIn.OnLowSignal:= OnLowSignalMeasurement;
  FCounter:= 0;
  Chart1.BottomAxis.Logarithmic:= True;
  Chart1.BottomAxis.Title.Caption:= 'Hz';
  Series1.Title:= 'Re(Z) Ohm ';
  Series2.Title:= 'Im(Z) Ohm ';
  btnStart.OnClick:= btnStopMeasurementClick;
  for i:= 0 to 1 do
    FSignalIn.Attenuator[i]:= 0;
  FSignalIn.Start;
end;

function Min(a, b: Integer): Integer;
begin
  if a < b then
    Result:= a
  else
    Result:= b;
end;

function TfmLCR.ShowFunction(const aData: TDataMatrix): Integer;
var i: Integer;
    dTime: Extended;
begin
  Series1.Clear;
  Series2.Clear;
  Series3.Clear;
  dTime:= 1E3/FSignalIn.Sampling;
  for i:= 0 to Min(Round(10/dTime), High(aData[0])) - 1 do
  if TstMod then begin
    Series1.AddXY(i*dTime, aData[0, i+1] - aData[0, i]);
    Series2.AddXY(i*dTime, aData[1, i+1] - aData[1, i]);
  end else begin
    Series1.AddXY(i*dTime, aData[0, i]);
    Series2.AddXY(i*dTime, aData[1, i]);
  end;
  Result:= 0;
end;

function TfmLCR.DataFunction(const aData: TDataMatrix): Integer;
const KOn = 10; 
var i, j, iU, iUr: Integer;
    aU, aUr: Com;
    aW: Rel;
    Omega, Omega1, Omega2,
    K2, SK2, SKx,
    SK2Omega2, SKyOmega1,
    SK2Omega_2, SKyOmega_1,
    Rx, Lx, Cx: Extended;
begin
  Result:= 0;
  if FCounter = 0 then
    for i:= 0 to cWavesCount do with FBuffer[i] do begin
      KZ:= Com0;
      W:= 0;
    end;
  Four1(@aData[0,0], Length(aData[0]) div 2, 1);
  Four1(@aData[1,0], Length(aData[1]) div 2, 1);
  if cbSwap.Checked then begin
    iU:= 1;
    iUr:= 0;
  end else begin
    iU:= 0;
    iUr:= 1;
  end;
  for i:= 0 to cWavesCount do with FBuffer[i] do begin
    j:= 1 shl (i + 1);
    aU:= cCom(aData[iU, j]);
    aUr:= cCom(aData[iUr, j]);
    aW:= cMod2(aU) * cMod2(cDiv(aUr,aU)) * cMod2(cDiv(cSub(aU, aUr), aU));
    KZ:= cAdd(KZ, cMuR(cDiv(aUr, cSub(aU, aUr)),aW));
    W:= W + aW;
  end;
  FCounter:= FCounter + 1;

  if DateTimeToTimeStamp(Time - FTime).Time < 1000 then
    Exit;

  aW:= 0;
  for i:= 0 to cWavesCount do with FBuffer[i] do begin
    KZ:= cCon(cMuR(KZ, 1/(W*FR)));
    aW:= aW + W; 
  end;
  aW:= aW / (cWavesCount + 1);

  SK2:= 0; SKx:= 0;
  SK2Omega2:= 0; SKyOmega1:= 0;
  SK2Omega_2:= 0; SKyOmega_1:= 0;

  Omega:= 2 * Pi * FSignalIn.Sampling / FSignalIn.Count;

  Series1.Clear;
  Series2.Clear;
  Series3.Clear;

  for i:= 0 to cWavesCount do with FBuffer[i] do begin
    Omega1:= Omega * (1 shl i);
    Omega2:= Sqr(Omega1);
    K2:= cMod2(KZ);
    SK2:= SK2 + W * K2;
    SKx:= SKx + W * KZ.x;
    SK2Omega2:= SK2Omega2 + W * K2 * Omega2;
    SKyOmega1:= SKyOmega1 + W * KZ.y * Omega1;
    SK2Omega_2:= SK2Omega_2 + W * K2 / Omega2;
    SKyOmega_1:= SKyOmega_1 + W * KZ.y / Omega1;

    // ForGraph:
    if W * KOn > aW then begin
      KZ:= cInv(KZ);
      Omega1:= Omega1 / (2*Pi);
      Series1.AddXY(Omega1, KZ.x);
      Series2.AddXY(Omega1, KZ.y);
      //Series3.AddXY(Omega1, Sqrt(Sqr(KZ.x)+Sqr(KZ.y)));
    end;
  end;
  Rx:= SKx / SK2;
  Lx:= - SKyOmega1 / SK2Omega2;
  Cx:= SK2Omega_2 / SKyOmega_1;

  if RelVer then
    lblR.Caption:= 'Rx= ' + ExtStr(Rx) + 'Ohm';

//  Series3.Title:= '|Z|';
  if Lx >= 0 then begin
    lblZ.Caption:= 'Lx= ' + ExtStr(Lx) + 'H';
    Series3.Title:= 'Lx';
    for i:= 0 to cWavesCount do if FBuffer[i].W * KOn > aW then
      Series3.AddXY(Omega*(1 shl i)/(2*Pi), Omega*(1 shl i)*Lx);
  end else if Cx >= 0 then begin
    lblZ.Caption:= 'Cx= ' + ExtStr(Cx) + 'F';
    Series3.Title:= 'Cx';
    for i:= 0 to cWavesCount do if FBuffer[i].W * KOn > aW then
      Series3.AddXY(Omega*(1 shl i)/(2*Pi), -1/(Omega*(1 shl i)*Cx));
  end else begin
    lblZ.Caption:= 'Lx, Cx= ?';
    Series3.Title:= ' ';
  end;
  FTime:= Time;
  FCounter:= 0;
end;

procedure TfmLCR.btnStartCalibrationClick(Sender: TObject);
begin
  FSignalIn.Stop;
  FSignalOut.Stop;
  cbBitsSamplingChange(Sender);
  OnShowMessage('Offset calibration...', True);
  FSignalIn.OnStop:= OnStopOffSetCalibr;
  FSignalIn.OnSaturation:= OnSaturationCalibration;
  FSignalIn.OnLowSignal:= nil;
  FSignalIn.MonitorFunction:= ShowFunction;
  Chart1.BottomAxis.Logarithmic:= False;
  Chart1.BottomAxis.Title.Caption:= 'ms';
  Series1.Title:= 'Channel 1';
  Series2.Title:= 'Channel 2';
  Series3.Title:= ' ';
  FSignalIn.CalibrateOffSet;
  btnStart.OnClick:= btnStopCalibrationClick;
end;

procedure TfmLCR.OnStart(Sender: TObject);
begin
  shLED.Brush.Color:= clRed;
  btnStart.Caption:= 'Stop';
  rgRunMode.Enabled:= False;
  cbIn.Enabled:= False;
  cbOut.Enabled:= False;
end;

procedure TfmLCR.OnStop(Sender: TObject);
begin
  shLED.Brush.Color:= clBtnFace;
  btnStart.Caption:= 'Start';
  rgRunMode.Enabled:= True;
  cbIn.Enabled:= True;
  cbOut.Enabled:= True;
end;

procedure TfmLCR.OnShowMessage(aMes: String; aState: Boolean);
begin
  if Trim(aMes) = '' then
    Exit;
  with reMes.SelAttributes do
    if aState then
      Color:= clWindowText
    else
      Color:= clRed;
  reMes.Lines.Add(aMes);
  reMes.SetFocus;
end;

procedure TfmLCR.btnStopMeasurementClick(Sender: TObject);
begin
  FSignalIn.Stop;
  FSignalOut.Stop;
  btnStart.OnClick:= btnStartMeasurementClick;
  OnShowMessage('Stop measurement.', True);
end;

procedure TfmLCR.btnStopCalibrationClick(Sender: TObject);
begin
  FSignalIn.OnStop:= OnStop;
  FSignalIn.Stop;
  FSignalOut.Stop;
  btnStart.OnClick:= btnStartCalibrationClick;
  OnShowMessage('Calibration aborted.', False);
end;

procedure TfmLCR.cbInChange(Sender: TObject);
var i: Integer;
begin
  FSignalIn.Device:= cbIn.ItemIndex;
  if FSignalIn.Device <> cbIn.ItemIndex then
    cbIn.ItemIndex:= FSignalIn.Device;
  cbInSrc.Clear;
  for i:= 0 to FSignalIn.SourcesNumber - 1 do
    cbInSrc.Items.Add(FSignalIn.SourceName[i]);
  cbInSrc.ItemIndex:= FSignalIn.Source;
end;

procedure TfmLCR.cbOutChange(Sender: TObject);
var i: Integer;
begin
  FSignalOut.Device:= cbOut.ItemIndex;
  if FSignalOut.Device <> cbOut.ItemIndex then
    cbOut.ItemIndex:= FSignalOut.Device;
  cbOutSrc.Clear;
  for i:= 0 to FSignalOut.SourcesNumber - 1 do
    cbOutSrc.Items.Add(FSignalOut.SourceName[i]);
  cbOutSrc.ItemIndex:= FSignalOut.Source;
end;

function TfmLCR.OnSaturationCalibration(aValue: Integer): Integer;
var adChannel: Integer;
begin
  if aValue = 0 then
    adChannel:= 1
  else
    adChannel:= 0;
  OnShowMessage('Saturation. Channel: ' + IntToStr(aValue), False);
  with FSignalOut do begin
    if Attenuator[aValue] <= Attenuator[adChannel] then
      Attenuator[aValue]:= Attenuator[aValue] + 1
    else
      Attenuator[adChannel]:= Attenuator[adChannel] + 1;
  end;
  Result:= 1;
end;

function TfmLCR.OnSaturationMeasurement(aValue: Integer): Integer;
begin
  OnShowMessage('Saturation. Channel: ' + IntToStr(aValue), False);
  if aValue < FSignalIn.AttNumber - 1 then
    FSignalIn.Attenuator[aValue]:= FSignalIn.Attenuator[aValue] + 1
  else
    OnSaturationCalibration(aValue);
  Result:= 1;
end;

function TfmLCR.OnLowSignalMeasurement(aValue: Integer): Integer;
var adChannel: Integer;
begin
  Result:= 1;
  //OnShowMessage('Low signal. Channel: ' + IntToStr(aValue), False);
  if FSignalIn.Attenuator[aValue] > 0 then
    FSignalIn.Attenuator[aValue]:= FSignalIn.Attenuator[aValue] - 1
  else begin
    if aValue = 0 then
      adChannel:= 1
    else
      adChannel:= 0;
    with FSignalOut do
      if Attenuator[aValue] >= Attenuator[adChannel] then
        if Attenuator[aValue] > 0 then
          Attenuator[aValue]:= Attenuator[aValue] - 1
        else
          Result:= 0
      else
        Attenuator[adChannel]:= Attenuator[adChannel] - 1;
  end;
end;

procedure TfmLCR.OnStopOffSetCalibr(Sender: TObject);
var i: Integer;
begin
  OnShowMessage('Amplitude calibration...', True);
  for i:= 0 to FSignalOut.Channels - 1 do
    FSignalOut.Attenuator[i]:= 0;
  if TstMod then
    FSignalOut.Trapezium(1E-3, 1E-4)
  else
    FSignalOut.Trapezium(1E-4, 1E-3);
  FSignalIn.OnStop:= OnStopAmplitudeCalibr;
  FSignalIn.CalibrateAmplitude(FSignalOut.Sampling/FSignalOut.Count);
end;

procedure TfmLCR.OnStopAmplitudeCalibr(Sender: TObject);
begin
  FSignalIn.Stop;
  FSignalOut.Stop;
  FSignalIn.OnStop:= OnStop;
  OnStop(Sender);
  btnStart.OnClick:= btnStartCalibrationClick;
  rgRunMode.ItemIndex:= 1;
  rgRunModeClick(Sender);
  OnShowMessage('Calibration done.', True);
end;

procedure TfmLCR.rgRunModeClick(Sender: TObject);
begin
  case rgRunMode.ItemIndex of
    0: begin
         btnStart.OnClick:= btnStartCalibrationClick;
         shCC.Visible:= True;
         if RelVer then
           lblR.Visible:= False;
         lblZ.Visible:= False;
         shpR.Pen.Style:= psDot;
         shpRGnd.Pen.Style:= psDot;
         shpRUp.Pen.Style:= psDot;
       end;
    1: begin
         btnStart.OnClick:= btnStartMeasurementClick;
         shCC.Visible:= False;
         if RelVer then
           lblR.Visible:= True;
         lblZ.Visible:= True;
         shpR.Pen.Style:= psSolid;
         shpRGnd.Pen.Style:= psSolid;
         shpRUp.Pen.Style:= psSolid;
       end;
  end;
end;

procedure TfmLCR.cbSwapClick(Sender: TObject);
begin
  if cbSwap.Checked then begin
    lblLeft.Left:= FlblRLeft;
    lblRight.Left:= FlblLLeft;
  end else begin
    lblLeft.Left:= FlblLLeft;
    lblRight.Left:= FlblRLeft;
  end;
end;

procedure TfmLCR.edRChange(Sender: TObject);
begin
  if edR.Text <> '' then
    FR:= StrToFloat(edR.Text);
end;

procedure TfmLCR.Chart1ClickAxis(Sender: TCustomChart; Axis: TChartAxis;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Axis = Sender.RightAxis then begin
    Sender.RightAxis.Grid.Visible:= True;
    Sender.LeftAxis.Grid.Visible:= False;
  end;
  if Axis = Sender.LeftAxis then begin
    Sender.RightAxis.Grid.Visible:= False;
    Sender.LeftAxis.Grid.Visible:= True;
  end;
end;

procedure TfmLCR.cbGraphScaleClick(Sender: TObject);
begin
  if cbGraphScale.Checked then begin
    Series2.VertAxis:= aLeftAxis;
    Series3.VertAxis:= aLeftAxis;
    Chart1.RightAxis.Grid.Visible:= False;
    Chart1.LeftAxis.Grid.Visible:= True;
    Chart1.LeftAxis.LabelsFont.Color:= clWindowText;
  end else begin
    Series2.VertAxis:= aRightAxis;
    Series3.VertAxis:= aRightAxis;
    Chart1.LeftAxis.LabelsFont.Color:= clRed;
  end;
end;

procedure TfmLCR.cbFixAxisClick(Sender: TObject);
begin
  if cbFixAxis.Checked then begin
    Chart1.LeftAxis.Automatic:= False;
    Chart1.RightAxis.Automatic:= False;
  end else begin
    Chart1.LeftAxis.Automatic:= True;
    Chart1.RightAxis.Automatic:= True;
  end;
end;

procedure TfmLCR.OnDelayTimer(Sender: TObject);
var i: Integer;
  aFile: TIniFile;
  aOK: Boolean;
begin
  DelayTimer.Enabled:= False;
  if not RelVer then begin
    OnShowMessage('SaveMixerToFile', True);
    SaveMixerToFile(ChangeFileExt(Paramstr(0),'.txt'));
  end;

  FSignalOut:= TSignalOut.Create(Self, aOK, OnShowMessage);
  if not aOK then
    Exit;
  cbOut.Clear;
  for i:= 0 to FSignalOut.DevicesNumber - 1 do
    cbOut.Items.Add(FSignalOut.DeviceName[i]);
  cbOut.ItemIndex:= FSignalOut.Device;
  cbOutChange(Self);

  FSignalIn:= TSignalIn.Create(Self, aOK, OnShowMessage);
  if not aOK then
    Exit;
  cbIn.Clear;
  for i:= 0 to FSignalIn.DevicesNumber - 1 do
    cbIn.Items.Add(FSignalIn.DeviceName[i]);
  cbIn.ItemIndex:= FSignalIn.Device;
  cbInChange(Self);
  FSignalOut.OnInvalidateOutput:= FSignalIn.InvalidateInput;
  FSignalIn.Output:= DataFunction;
  FSignalIn.OnStart:= OnStart;
  FSignalIn.OnStop:= OnStop;
  FSignalIn.Count:= FSignalIn.Sampling div 10;
  if (cbIn.Items.Count = 0) or (cbOut.Items.Count = 0) then begin
    btnStart.Enabled:= False;
    OnShowMessage('Sound card not found.', False);
  end;
  aFile:= TIniFile.Create(ChangeFileExt(Paramstr(0),'.ini'));
  FSignalOut.Device:= aFile.ReadInteger(sAppSection, 'Output', FSignalOut.Device);
  cbOut.ItemIndex:= FSignalOut.Device;
  cbOutChange(Self);
  FSignalOut.Source:= aFile.ReadInteger(sAppSection, 'SourceOut', FSignalOut.Source);
  cbOutSrc.ItemIndex:= FSignalOut.Source;
  cbOutSrcChange(Self);
  FSignalIn.Device:= aFile.ReadInteger(sAppSection, 'Input', FSignalIn.Device);
  cbIn.ItemIndex:= FSignalIn.Device;
  cbInChange(Self);
  FSignalIn.Source:= aFile.ReadInteger(sAppSection, 'SourceIn', FSignalIn.Source);
  cbInSrc.ItemIndex:= FSignalIn.Source;
  cbInSrcChange(Self);
  FR:= aFile.ReadFloat(sAppSection, 'Reference resistor', FR);
  edR.Text:= FloatToStr(FR);
  cbSwap.Checked:= aFile.ReadBool(sAppSection, 'Swap L<->R', cbSwap.Checked);
  cbSwapClick(Self);
  if RelVer then begin
    rgRunMode.ItemIndex:= aFile.ReadInteger(sAppSection, 'Run mode', rgRunMode.ItemIndex);
    rgRunModeClick(Self);
    FSignalIn.LoadIni(aFile);
  end;
  cbBits.ItemIndex:= FSignalIn.Bits div 8 - 1;
  FSignalOut.Bits:= FSignalIn.Bits;
  case FSignalIn.Sampling of
    11025: cbSampling.ItemIndex:= 0;
    22050: cbSampling.ItemIndex:= 1;
    44100: cbSampling.ItemIndex:= 2;
    48000: cbSampling.ItemIndex:= 3;
    96000: cbSampling.ItemIndex:= 4;
   192000: cbSampling.ItemIndex:= 5;
  end;
  FSignalOut.Sampling:= FSignalIn.Sampling;
  cbBits.ItemIndex:= aFile.ReadInteger(sAppSection, 'IdBits', cbBits.ItemIndex);
  cbSampling.ItemIndex:= aFile.ReadInteger(sAppSection, 'IdSampling', cbSampling.ItemIndex);
  aFile.Free;
  cbBitsSamplingChange(Self);
  OnShowMessage('- Creation is done. -', True);
  btnStart.Enabled:= True;
end;

procedure TfmLCR.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var aFile: TIniFile;
begin
  CanClose:= False;
  if not btnStart.Enabled then
    Exit;
  if not FCanClose then try
    aFile:= TIniFile.Create(ChangeFileExt(Paramstr(0),'.ini'));
    aFile.WriteInteger(sAppSection, 'Output', FSignalOut.Device);
    aFile.WriteInteger(sAppSection, 'SourceOut', FSignalOut.Source);
    aFile.WriteInteger(sAppSection, 'Input', FSignalIn.Device);
    aFile.WriteInteger(sAppSection, 'SourceIn', FSignalIn.Source);
    aFile.WriteInteger(sAppSection, 'IdBits', cbBits.ItemIndex);
    aFile.WriteInteger(sAppSection, 'IdSampling', cbSampling.ItemIndex);
    aFile.WriteFloat(sAppSection, 'Reference resistor', FR);
    aFile.WriteBool(sAppSection, 'Swap L<->R', cbSwap.Checked);
    aFile.WriteInteger(sAppSection, 'Run mode', rgRunMode.ItemIndex);
    if RelVer then
      FSignalIn.SaveIni(aFile);
    aFile.Free;
    FreeAndNil(FSignalIn);
    FreeAndNil(FSignalOut);
    reMes.BringToFront;
    reMes.Align:= alClient;
    reMes.SetFocus;
    OnShowMessage('- Destruction is done. -', True);
  except
    FCanClose:= True;
  end;
  DecimalSeparator:= OldDecSep;
  if RelVer then
    FCanClose:= True;
  CanClose:= FCanClose;
  FCanClose:= True;
end;

procedure TfmLCR.cbOutSrcChange(Sender: TObject);
begin
  FSignalOut.Source:= cbOutSrc.ItemIndex;
  cbOutSrc.ItemIndex:= FSignalOut.Source;
end;

procedure TfmLCR.cbInSrcChange(Sender: TObject);
begin
  FSignalIn.Source:= cbInSrc.ItemIndex;
  cbInSrc.ItemIndex:= FSignalIn.Source;
end;

procedure TfmLCR.cbBitsSamplingChange(Sender: TObject);
begin
  FSignalOut.Bits:= StrToInt(cbBits.Items[cbBits.ItemIndex]);
  FSignalIn.Bits:= FSignalOut.Bits;
  FSignalOut.Sampling:= StrToInt(cbSampling.Items[cbSampling.ItemIndex]);
  FSignalIn.Sampling:= FSignalOut.Sampling;
end;

end.
