unit uTest;

interface

uses
  uCommon, uWaveIn, uWaveOut, //uMixerControls,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TeEngine, Series, ExtCtrls, TeeProcs, Chart;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    Chart1: TChart;
    btnStartIn: TButton;
    btnStopIn: TButton;
    Series1: TFastLineSeries;
    Series2: TFastLineSeries;
    cbDevicesIn: TComboBox;
    cbDevicesOut: TComboBox;
    btnStartOut: TButton;
    btnStopOut: TButton;
    Label1: TLabel;
    Label2: TLabel;
    edDel: TEdit;
    lblDelta: TLabel;
    Button1: TButton;
    cbDirect: TCheckBox;
    cbSync: TCheckBox;
    cbDeriv: TCheckBox;
    edSampling: TEdit;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbDevicesInChange(Sender: TObject);
    procedure btnStartInClick(Sender: TObject);
    procedure btnStopInClick(Sender: TObject);
    procedure cbDevicesOutChange(Sender: TObject);
    procedure btnStartOutClick(Sender: TObject);
    procedure btnStopOutClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Dclarations prives }
    FTime: TDateTime;
    FWaveIn: TWaveIn;
    FWaveOut: TWaveOut;
    procedure UpdateFlags;
    function DataFunction(const aData: TDataMatrix): Integer;    
    procedure MesProc(aMes: String; aState: Boolean);
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

//const cSampling = 11025*2;// 48000;

uses MMSystem;

{$R *.DFM}

procedure TForm1.UpdateFlags;
begin
  OpenFlags:= cOpenFlags;
  if cbDirect.Checked then
    OpenFlags:= OpenFlags or WAVE_FORMAT_DIRECT;
  if cbSync.Checked then
    OpenFlags:= OpenFlags or WAVE_ALLOWSYNC;
end;

procedure TForm1.MesProc(aMes: String; aState: Boolean);
begin
  Memo1.Lines.Add(aMes);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  aOK: Boolean;
  i: Integer;
begin
   FTime:= Time;
   // Input
   FWaveIn:= TWaveIn.Create(Self, aOK, MesProc);
   FWaveIn.DataFunction:= DataFunction;
   FWaveIn.Count:= FWaveIn.Sampling div 100;
   for i:= 0 to FWaveIn.DevicesNumber - 1 do
     cbDevicesIn.Items.Add(FWaveIn.DeviceName[i]);
   cbDevicesIn.ItemIndex:= FWaveIn.Device;
   // Output
   FWaveOut:= TWaveOut.Create(Self, aOK, MesProc);
   for i:= 0 to FWaveOut.DevicesNumber - 1 do
     cbDevicesOut.Items.Add(FWaveOut.DeviceName[i]);
   cbDevicesOut.ItemIndex:= FWaveOut.Device;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FWaveOut);
  FreeAndNil(FWaveIn);
end;

procedure TForm1.cbDevicesInChange(Sender: TObject);
begin
  FWaveIn.Device:= cbDevicesIn.ItemIndex;
  cbDevicesIn.ItemIndex:= FWaveIn.Device;
end;

function TForm1.DataFunction(const aData: TDataMatrix): Integer;
var i: Integer;
begin
  Result:= 0;
  if DateTimeToTimeStamp(FTIme - Time).Time < 50 then
    Exit;
  //if aData[0,0] > 0 then Exit;
  Series1.Clear;
  Series2.Clear;
  if cbDeriv.Checked then for i:= 0 to High(aData[0]) - 1 do begin
    Series1.AddXY(i, aData[0,i+1] - aData[0,i]);
    Series2.AddXY(i+StrToFloat(edDel.Text), aData[1,i+1] - aData[1,i]);
  end else for i:= 0 to High(aData[0]) do begin
    Series1.AddXY(i, aData[0,i]);
    Series2.AddXY(i+StrToFloat(edDel.Text), aData[1,i]);
  end;

  FTIme:= Time;
end;

procedure TForm1.btnStartInClick(Sender: TObject);
begin
  UpdateFlags;
  FWaveIn.Sampling:= StrToInt(edSampling.Text);
  FWaveIn.Count:= FWaveOut.Count*10;
  FWaveIn.Open;
end;

procedure TForm1.btnStopInClick(Sender: TObject);
begin
  FWaveIn.Close;
end;

procedure TForm1.cbDevicesOutChange(Sender: TObject);
begin
  FWaveOut.Device:= cbDevicesOut.ItemIndex;
  cbDevicesOut.ItemIndex:= FWaveOut.Device;
end;

procedure TForm1.btnStartOutClick(Sender: TObject);
begin
  UpdateFlags;
  FWaveOut.Sampling:= StrToInt(edSampling.Text);
  FWaveOut.Trapezium(1E-4, 80/FWaveOut.Sampling);
end;

procedure TForm1.btnStopOutClick(Sender: TObject);
begin
  FWaveOut.Close;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FWaveIn.InvalidateInput(Sender);
end;

end.
