unit uCommon;

interface

uses
  MMSystem, Controls, Classes, Windows;

const
  {$EXTERNALSYM WAVE_FORMAT_DIRECT}
  WAVE_FORMAT_DIRECT = $0008;
  AllocFlags = GMEM_FIXED or GMEM_DDESHARE or GMEM_NOCOMPACT or GMEM_NODISCARD;
  RelVer =  True;  //  False;
  TstMod =  False;  //  True
  cMaxSampling = 200000;//50000;
type
  TDataVector = array of Extended;
  TDataMatrix = array of TDataVector;

  TIntFunction = function(aValue: Integer): Integer of object;
  TMessageProc = procedure(aMes: String; aState: Boolean) of object;
  TDataFunction = function(const aData: TDataMatrix): Integer of object;

  TMMWinControl = class(TWinControl)
  private
    FMessageProc: TMessageProc;
  protected
    FObjectName: String;
    FDeviceID: LongWord;
    FDeviceHD: Integer;
    FError: MMRESULT;
    FIsOpen: Boolean;    
    procedure ErrMes(aMes: String);
    procedure InfMes(aMes: String);
  public
    constructor Create(AOwner: TComponent;
                       aMessageProc: TMessageProc = nil); reintroduce; 
    property MessageProc: TMessageProc write FMessageProc;
  end;

  TSignalProcessing = class
  protected
    FDestinations: array of TDataFunction;
    FMessageProc: TMessageProc;
    procedure AddDestination(aDestination: TDataFunction); virtual;
  public
    procedure Clear;
    function Input(const aData: TDataMatrix): Integer; virtual;
    property Output: TDataFunction write AddDestination;
    property MessageProc: TMessageProc write FMessageProc;
  end;

procedure SetMaxFormatEx(aFormats: Cardinal; var aFormatEx: TWaveFormatEx);
procedure UpdateFormatEx(var aFormatEx: TWaveFormatEx);

procedure Proc8(var a; aValue: Extended);
procedure Proc16(var a; aValue: Extended);
function Func8(var a): Extended;
function Func16(var a): Extended;

function LineInfo(aLine: pMixerLine): String;
function CtrlInfo(aCtrl: pMIXERCONTROL): String;

implementation

uses
  Sysutils, Forms;

procedure SetMaxFormatEx(aFormats: LongWord; var aFormatEx: TWaveFormatEx);
begin
  with aFormatEx do begin
    nSamplesPerSec:= 11025;
    while aFormats shr 4 > 0 do begin
      aFormats:= aFormats shr 4;
      nSamplesPerSec:= nSamplesPerSec * 2;
      if nSamplesPerSec >= 44100 then
        Break;
    end;
    if aFormats shr 2 > 0 then begin
      wBitsPerSample:= 16;
      aFormats:= aFormats shr 2;
    end else
      wBitsPerSample:= 8;
    if aFormats > 1 then
      nChannels:= 2
    else
      nChannels:= 1;
  end;
end;

procedure UpdateFormatEx(var aFormatEx: TWaveFormatEx);
begin
  with aFormatEx do begin
    nBlockAlign:= (wBitsPerSample div 8) * nChannels;
    nAvgBytesPerSec:= nSamplesPerSec * nBlockAlign;
    wFormatTag:= WAVE_FORMAT_PCM;
    cbSize:= 0;
  end;
end;

procedure Proc8(var a; aValue: Extended);
const cRange = 0.5 * (High(Byte) - Low(Byte));
var aVal: Byte absolute a;
begin
  aVal:= Round(aValue * cRange + cRange);
end;

procedure Proc16(var a; aValue: Extended);
const cRange = 0.5 * (High(SmallInt) - Low(SmallInt));
var aVal: SmallInt absolute a;
begin
  aVal:= Round(aValue * cRange);
end;

function Func8(var a): Extended;
const cRange = 2 / (High(Byte) - Low(Byte));
var aVal: Byte absolute a;
begin
  Result:= (aVal - 127.5) * cRange;
end;

function Func16(var a): Extended;
const cRange = 2 / (High(SmallInt) - Low(SmallInt));
var aVal: SmallInt absolute a;
begin
  Result:= aVal * cRange;
end;

constructor TMMWinControl.Create(AOwner: TComponent;
                           aMessageProc: TMessageProc = nil);
begin
  inherited Create(AOwner);
  FMessageProc:= aMessageProc;
  ParentWindow:= Application.Handle;
end;

procedure TMMWinControl.ErrMes(aMes: String);
begin
  if Assigned(FMessageProc) then
    FMessageProc(FObjectName + ': ' + aMes, False);
end;

procedure TMMWinControl.InfMes(aMes: String);
begin
  if Assigned(FMessageProc) then
    FMessageProc(FObjectName + ': ' + aMes, True);
end;

procedure TSignalProcessing.AddDestination(aDestination: TDataFunction);
begin
  SetLength(FDestinations, Succ(Length(FDestinations)));
  FDestinations[High(FDestinations)]:= aDestination;
end;

procedure TSignalProcessing.Clear;
begin
  FDestinations:= nil;
end;

function TSignalProcessing.Input(const aData: TDataMatrix): Integer;
var i: Integer;
begin
  Result:= 0;
  for i:= 0 to High(FDestinations) do begin
    Result:= FDestinations[i](aData);
    if Result <> 0 then
      Exit;
  end;
end;

function LineInfo(aLine: pMixerLine): String;
var St: String;
begin
  St:= '"' + aLine.szName + '": ';
  case aLine.Target.dwType of
    MIXERLINE_TARGETTYPE_AUX:
      St:= St + '[AUX]';
    MIXERLINE_TARGETTYPE_MIDIIN:
      St:= St + '[MIDIIN]';
    MIXERLINE_TARGETTYPE_MIDIOUT:
      St:= St + '[MIDIOUT]';
    MIXERLINE_TARGETTYPE_UNDEFINED:
      St:= St + '[UNDEFINED]';
    MIXERLINE_TARGETTYPE_WAVEIN:
      St:= St + '[WAVEIN]';
    MIXERLINE_TARGETTYPE_WAVEOUT:
      St:= St + '[WAVEOUT]';
  end;
  St:= St + ' ';
  case aLine.dwComponentType of
    // Destinations
    MIXERLINE_COMPONENTTYPE_DST_DIGITAL: St:= St + '<DST_DIGITAL>';
    MIXERLINE_COMPONENTTYPE_DST_HEADPHONES: St:= St + '<DST_HEADPHONES>';
    MIXERLINE_COMPONENTTYPE_DST_LINE: St:= St + '<DST_LINE>';
    MIXERLINE_COMPONENTTYPE_DST_MONITOR: St:= St + '<DST_MONITOR>';
    MIXERLINE_COMPONENTTYPE_DST_SPEAKERS: St:= St + '<DST_SPEAKERS>';
    MIXERLINE_COMPONENTTYPE_DST_TELEPHONE: St:= St + '<DST_TELEPHONE>';
    MIXERLINE_COMPONENTTYPE_DST_UNDEFINED: St:= St + '<DST_UNDEFINED>';
    MIXERLINE_COMPONENTTYPE_DST_VOICEIN: St:= St + '<DST_VOICEIN>';
    MIXERLINE_COMPONENTTYPE_DST_WAVEIN: St:= St + '<DST_WAVEIN>';
    // Sources
    MIXERLINE_COMPONENTTYPE_SRC_ANALOG: St:= St + '<SRC_ANALOG>';
    MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY: St:= St + '<SRC_AUXILIARY>';
    MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC: St:= St + '<SRC_COMPACTDISC>';
    MIXERLINE_COMPONENTTYPE_SRC_DIGITAL: St:= St + '<SRC_DIGITAL>';
    MIXERLINE_COMPONENTTYPE_SRC_LINE: St:= St + '<SRC_LINE>';
    MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE: St:= St + '<SRC_MICROPHONE>';
    MIXERLINE_COMPONENTTYPE_SRC_PCSPEAKER: St:= St + '<SRC_PCSPEAKER>';
    MIXERLINE_COMPONENTTYPE_SRC_SYNTHESIZER: St:= St + '<SRC_SYNTHESIZER>';
    MIXERLINE_COMPONENTTYPE_SRC_TELEPHONE: St:= St + '<SRC_TELEPHONE>';
    MIXERLINE_COMPONENTTYPE_SRC_UNDEFINED: St:= St + '<SRC_UNDEFINED>';
    MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT: St:= St + '<SRC_WAVEOUT>';
  end;
  St:= St + ' Ch= ' + IntToStr(aLine.cChannels)
       + ' Ctrls= ' + IntToStr(aLine.cControls);
  Result:= St;
end;

function CtrlInfo(aCtrl: PMIXERCONTROL): String;
var St: String;
begin
  St:= '"' + aCtrl.szName + '" ';
  case aCtrl.dwControlType of
    // MIXERCONTROL_CT_CLASS_CUSTOM
    MIXERCONTROL_CONTROLTYPE_CUSTOM: St:= St + '<CUSTOM>';

    // MIXERCONTROL_CT_CLASS_FADER
    MIXERCONTROL_CONTROLTYPE_BASS: St:= St + '<BASS>';
    MIXERCONTROL_CONTROLTYPE_EQUALIZER: St:= St + '<EQUALIZER>';
    MIXERCONTROL_CONTROLTYPE_FADER: St:= St + '<FADER>';
    MIXERCONTROL_CONTROLTYPE_TREBLE: St:= St + '<TREBLE>';
    MIXERCONTROL_CONTROLTYPE_VOLUME: St:= St + '<VOLUME>';

    // MIXERCONTROL_CT_CLASS_LIST
    MIXERCONTROL_CONTROLTYPE_MIXER: St:= St + '<MIXER>';
    MIXERCONTROL_CONTROLTYPE_MULTIPLESELECT: St:= St + '<MULTIPLESELECT>';
    MIXERCONTROL_CONTROLTYPE_MUX: St:= St + '<MUX>';
    MIXERCONTROL_CONTROLTYPE_SINGLESELECT: St:= St + '<SINGLESELECT>';

    // MIXERCONTROL_CT_CLASS_METER
    MIXERCONTROL_CONTROLTYPE_BOOLEANMETER: St:= St + '<BOOLEANMETER>';
    MIXERCONTROL_CONTROLTYPE_PEAKMETER: St:= St + '<PEAKMETER>';
    MIXERCONTROL_CONTROLTYPE_SIGNEDMETER: St:= St + '<SIGNEDMETER>';
    MIXERCONTROL_CONTROLTYPE_UNSIGNEDMETER: St:= St + '<UNSIGNEDMETER>';

    // MIXERCONTROL_CT_CLASS_NUMBER
    MIXERCONTROL_CONTROLTYPE_DECIBELS: St:= St + '<DECIBELS>';
    MIXERCONTROL_CONTROLTYPE_PERCENT: St:= St + '<PERCENT>';
    MIXERCONTROL_CONTROLTYPE_SIGNED: St:= St + '<SIGNED>';
    MIXERCONTROL_CONTROLTYPE_UNSIGNED: St:= St + '<UNSIGNED>';

    // MIXERCONTROL_CT_CLASS_SLIDER
    MIXERCONTROL_CONTROLTYPE_PAN: St:= St + '<PAN>';
    MIXERCONTROL_CONTROLTYPE_QSOUNDPAN: St:= St + '<QSOUNDPAN>';
    MIXERCONTROL_CONTROLTYPE_SLIDER: St:= St + '<SLIDER>';

    // MIXERCONTROL_CT_CLASS_SWITCH
    MIXERCONTROL_CONTROLTYPE_BOOLEAN: St:= St + '<BOOLEAN>';
    MIXERCONTROL_CONTROLTYPE_BUTTON: St:= St + '<BUTTON>';
    MIXERCONTROL_CONTROLTYPE_LOUDNESS: St:= St + '<LOUDNESS>';
    MIXERCONTROL_CONTROLTYPE_MONO: St:= St + '<MONO>';
    MIXERCONTROL_CONTROLTYPE_MUTE: St:= St + '<MUTE>';
    MIXERCONTROL_CONTROLTYPE_ONOFF: St:= St + '<ONOFF>';
    MIXERCONTROL_CONTROLTYPE_STEREOENH: St:= St + '<STEREOENH>';

    //MIXERCONTROL_CT_CLASS_TIME
    MIXERCONTROL_CONTROLTYPE_MICROTIME: St:= St + '<MICROTIME>';
    MIXERCONTROL_CONTROLTYPE_MILLITIME: St:= St + '<MILLITIME>';
  end;
  St:= St + ' |';
  if (aCtrl.fdwControl and MIXERCONTROL_CONTROLF_DISABLED) = MIXERCONTROL_CONTROLF_DISABLED then
    St:= St + ' DISABLED ';
  if (aCtrl.fdwControl and MIXERCONTROL_CONTROLF_MULTIPLE) = MIXERCONTROL_CONTROLF_MULTIPLE then
    St:= St + ' MULTIPLE ';
  if (aCtrl.fdwControl and MIXERCONTROL_CONTROLF_UNIFORM) = MIXERCONTROL_CONTROLF_UNIFORM then
    St:= St + ' UNIFORM ';
  St:= St + '| Multi= ' + IntToStr(aCtrl.cMultipleItems)
    + ' ID= ' + IntToStr(aCtrl.dwControlID);
  Result:= St;
end;

end.
