unit uMixerControls;

interface

uses
  uCommon, Classes, Messages, MMSystem;

type
  TCtrlDataArray = array of Byte;
  TMixerControls = class(TMMWinControl)
  private
    FOnChangeControl: TNotifyEvent;
    FControls: array of record
      Active: Boolean;
      Details: TMIXERCONTROLDETAILS;
      Channels: Integer;
      ItemsNmb: Integer;
      ControlType: LongWord;
      SavedData,
      CurrentData,
      QueryData: TCtrlDataArray;
      CheckData: array [0..1] of TCtrlDataArray;
    end;
    procedure OnChange(var aMes : TMessage); message MM_MIXM_CONTROL_CHANGE;
    function ControlInRange(aIndex: Integer; aProcName: String): Boolean;
  protected
    procedure Init; virtual;
    function AddControl(aControl: PMIXERCONTROL;
                        aChannels: Integer): Integer;
    function CaptureControl(aIndex: Integer): Boolean;
    function LeaveControl(aIndex: Integer): Boolean;
    function SetValue(aIndex, aChannel, aItemIndex: Integer;
                       aValue: Pointer): Boolean;
    function SysErrMes: String;
    function IsCaptured(aIndex: Integer): Boolean;
  public
    constructor Create(AOwner: TComponent; aMessageProc: TMessageProc = nil); virtual;
    destructor Destroy; override;
    property OnChangeControl: TNotifyEvent write FOnChangeControl;
  end;

implementation

uses
  Forms, Windows, Sysutils;


const
  cItemSize = 4;

procedure TMixerControls.Init;
var i: Integer;
begin
  for i:= High(FControls) downto 0 do with FControls[i] do
    LeaveControl(i);
  FControls:= nil;
  {$B-}
  if FIsOpen then begin
    FError:= mixerClose(FDeviceHD);
    if FError = MMSYSERR_NOERROR then
      FIsOpen:= False
    else
      ErrMes('Init: Can not Close: ' + SysErrMes + '.');
  end;
  FDeviceID:= High(FDeviceID);
end;

constructor TMixerControls.Create(AOwner: TComponent; aMessageProc: TMessageProc = nil);
begin
  inherited Create(AOwner, aMessageProc);
  FControls:= nil;
  FDeviceHD:= 0;
  FIsOpen:= False;
  Init;
end;

destructor TMixerControls.Destroy;
begin
  Init;
  inherited;
end;

function TMixerControls.ControlInRange(aIndex: Integer; aProcName: String): Boolean;
begin
  if (aIndex < 0) or (aIndex > High(FControls)) then begin
    Result:= False;
    ErrMes(aProcName + ': Control index ' + IntToStr(aIndex) + ' is out of range.');
  end else
    Result:= True;
end;

function TMixerControls.IsCaptured(aIndex: Integer): Boolean;
var i: Integer;
begin
  Result:= False;
  try
    if not ControlInRange(aIndex, 'IsCaptured') then
      Exit;
    with FControls[aIndex] do begin
      if Active then begin
        Result:= True;
        Exit;
      end;
      Details.paDetails:= @CurrentData[0];
      FError:= mixerGetControlDetails(FDeviceHD, @Details, MIXER_GETCONTROLDETAILSF_VALUE);
      if FError <> MMSYSERR_NOERROR then begin
        ErrMes('IsCaptured: Get Current Details: ' + SysErrMes);
        Exit;
      end;
      for i:= Low(CheckData) to High(CheckData) do begin
        Details.paDetails:= @CheckData[i, 0];
        FError:= mixerSetControlDetails(FDeviceHD, @Details, MIXER_SETCONTROLDETAILSF_VALUE);
        if FError <> MMSYSERR_NOERROR then begin
          ErrMes('IsCaptured: Set Check Details ' + IntToStr(i) + ': ' + SysErrMes);
          Exit;
        end;
        Sleep(10); Application.ProcessMessages;
//        Sleep(10); Application.ProcessMessages;
        Details.paDetails:= @QueryData[0];
        FError:= mixerGetControlDetails(FDeviceHD, @Details, MIXER_GETCONTROLDETAILSF_VALUE);
        if FError <> MMSYSERR_NOERROR then begin
          ErrMes('IsCaptured: Get Query Details: ' + SysErrMes);
          Exit;
        end;
        if not CompareMem(@CheckData[i, 0], @QueryData[0], Length(QueryData)) then begin
          Result:= True;
          Exit;
        end;
      end;
    end;
  except
    ErrMes('IsCaptured: Exception.');
  end;
end;

procedure TMixerControls.OnChange(var aMes : TMessage);
var i: Integer;
begin
  {$B-}
  if HMIXER(aMes.WParam) <> FDeviceHD then
    Exit;
  try
    for i:= 0 to High(FControls) do with FControls[i] do
      if Active and (DWORD(aMes.LParam) = Details.dwControlID) then begin
        Details.paDetails:= @QueryData[0];
        FError:= mixerGetControlDetails(FDeviceHD, @Details, MIXER_GETCONTROLDETAILSF_VALUE);
        if FError <> MMSYSERR_NOERROR then begin
          ErrMes('OnChange: Get Query Details: ' + SysErrMes);
          Exit;
        end;
        if not CompareMem(@CurrentData[0], @QueryData[0], Length(CurrentData))
          then begin
            Details.paDetails:= @CurrentData[0];
            FError:= mixerSetControlDetails(FDeviceHD, @Details, MIXER_SETCONTROLDETAILSF_VALUE);
            if FError <> MMSYSERR_NOERROR then begin
              ErrMes('OnChange: Set Current Details: ' + SysErrMes);
              Exit;
            end;
            if Assigned(FOnChangeControl) then
              FOnChangeControl(Self);
          end;
        Break;
      end;
  except
    ErrMes('OnChange: Exception.');
  end;
end;

function TMixerControls.SetValue(aIndex, aChannel, aItemIndex: Integer;
                                  aValue: Pointer): Boolean;
begin
  Result:= False;
  if not ControlInRange(aIndex, 'SetValue') then
    Exit;
  with FControls[aIndex] do begin
    if (aChannel < 0) or (Channels <= aChannel) then begin
      ErrMes('SetValue: Channel index is out of range.');
      Exit;
    end;
    if (aItemIndex < 0) or (ItemsNmb <= aItemIndex) then begin
      ErrMes('SetValue: Item index is out of range.');
      Exit;
    end;
  end;
  try
    with FControls[aIndex] do begin
      if ControlType = MIXERCONTROL_CONTROLTYPE_MUX then
        ZeroMemory(@CurrentData[0], Length(CurrentData));
      Move(aValue^,
        CurrentData[(aChannel*ItemsNmb+aItemIndex)*cItemSize], cItemSize);
      Details.paDetails:= @CurrentData[0];
      FError:= mixerSetControlDetails(FDeviceHD, @Details,
                                      MIXER_SETCONTROLDETAILSF_VALUE);
      if FError <> MMSYSERR_NOERROR then begin
        ErrMes('SetValue: Set Current Details: ' + SysErrMes);
        Exit;
      end;
      if Assigned(FOnChangeControl) then
        FOnChangeControl(Self);
    end;
    Result:= True;
  except
    ErrMes('SetValue: Exception.');
  end;
end;

function TMixerControls.AddControl(aControl: PMIXERCONTROL;
                                   aChannels: Integer): Integer;
var aSize, i: Integer;
    aMixUns: MIXERCONTROLDETAILS_UNSIGNED;
    aMixBool: MIXERCONTROLDETAILS_BOOLEAN;
begin
  {$B-}
  Result:= -1;
  try
    SetLength(FControls, Length(FControls) + 1);
    with FControls[High(FControls)] do begin
      ZeroMemory(@Details, SizeOf(Details));
      Active:= False;
      ControlType:= aControl.dwControlType;
      if (aControl.fdwControl and MIXERCONTROL_CONTROLF_UNIFORM)
        = MIXERCONTROL_CONTROLF_UNIFORM then
          Channels:= 1
      else
        Channels:= aChannels;
      if (aControl.fdwControl and MIXERCONTROL_CONTROLF_MULTIPLE)
        = MIXERCONTROL_CONTROLF_MULTIPLE then begin
          Details.cMultipleItems:= aControl.cMultipleItems;
          ItemsNmb:= aControl.cMultipleItems;
        end else begin
          Details.cMultipleItems := 0;
          ItemsNmb:= 1;
        end;
      Details.cbStruct:= SizeOf(Details);
      Details.cbDetails:= cItemSize;
      Details.dwControlID:= aControl.dwControlID;
      Details.cChannels:= Channels;
      aSize:= ItemsNmb * cItemSize * Channels;
      SetLength(SavedData, aSize);
      ZeroMemory(@SavedData[0], aSize);
      SetLength(CurrentData, aSize);
      ZeroMemory(@CurrentData[0], aSize);
      SetLength(QueryData, aSize);
      ZeroMemory(@QueryData[0], aSize);
  // Data for capture check
      for i:= Low(CheckData) to High(CheckData) do begin
        SetLength(CheckData[i], aSize);
        ZeroMemory(@CheckData[i, 0], aSize);
      end;
      case ControlType of
        MIXERCONTROL_CONTROLTYPE_MUX, MIXERCONTROL_CONTROLTYPE_MIXER: begin
          aMixBool.fValue:= 1;
          Move(aMixBool, CheckData[0, 0], cItemSize);
          Move(aMixBool, CheckData[1, cItemSize], cItemSize);
        end;
        MIXERCONTROL_CONTROLTYPE_VOLUME: begin
          aMixUns.dwValue:= High(WORD);
          Move(aMixUns, CheckData[1, 0], cItemSize);
        end;
        MIXERCONTROL_CONTROLTYPE_MUTE: begin
          aMixBool.fValue:= 1;
          Move(aMixBool, CheckData[1, 0], cItemSize);
        end;
      end;
  //
      Details.paDetails:= @SavedData[0];
      FError:= mixerGetControlDetails(FDeviceHD, @Details,
                                      MIXER_GETCONTROLDETAILSF_VALUE);
      if FError <> MMSYSERR_NOERROR then begin
        ErrMes('AddControl: Get Saved Details: ' + SysErrMes +
               ', Control info: ' + CtrlInfo(aControl));
        Exit;
      end;
      Move(SavedData[0], CurrentData[0], Length(SavedData));
    end;
    Result:= High(FControls);
  except
    ErrMes('AddControl: Exception.');
  end;
end;

function TMixerControls.CaptureControl(aIndex: Integer): Boolean;
begin
  Result:= False;
  if not ControlInRange(aIndex, 'CaptureControl') then
    Exit;
  try
    with FControls[aIndex] do begin
      if Active then begin
        Result:= True;
        Exit;
      end;
      if IsCaptured(aIndex) then begin
        ErrMes('CaptureControl: Captured by another process.');
        Exit;
      end;
      Active:= True;
      Details.paDetails:= @CurrentData[0];
      FError:= mixerSetControlDetails(FDeviceHD, @Details,
                                      MIXER_SETCONTROLDETAILSF_VALUE);
      if FError <> MMSYSERR_NOERROR then begin
        ErrMes('CaptureControl: Set Current Details: ' + SysErrMes);
        Exit;
      end;
      Result:= Active;
    end;
  except
    ErrMes('CaptureControl: Exception.');
  end;
end;

function TMixerControls.LeaveControl(aIndex: Integer): Boolean;
begin
  Result:= False;
  if not ControlInRange(aIndex, 'LeaveControl') then
    Exit;
  try
    with FControls[aIndex] do begin
      if not Active then begin
        Result:= True;
        Exit;
      end;
      Active:= False;
      Details.paDetails:= @SavedData[0];
      FError:= mixerSetControlDetails(FDeviceHD, @Details,
                                      MIXER_SETCONTROLDETAILSF_VALUE);
      if FError <> MMSYSERR_NOERROR then begin
        ErrMes('LeaveControl: Set Saved Details: ' + SysErrMes);
        Exit;
      end;
    end;
    Result:= True;
  except
    ErrMes('LeaveControl: Exception.');
  end;
end;

function TMixerControls.SysErrMes: String;
begin
  Result:= '';
  case FError of
    MMSYSERR_NOERROR: Exit;
    MIXERR_INVALLINE: Result:= 'MIXERR_INVALLINE';
    MIXERR_INVALCONTROL: Result:= 'MIXERR_INVALCONTROL';
    MIXERR_INVALVALUE: Result:= 'MIXERR_INVALVALUE';
    MMSYSERR_BADDEVICEID: Result:= 'MMSYSERR_BADDEVICEID';
    MMSYSERR_INVALFLAG: Result:= 'MMSYSERR_INVALFLAG';
    MMSYSERR_INVALHANDLE: Result:= 'MMSYSERR_INVALHANDLE';
    MMSYSERR_INVALPARAM: Result:= 'MMSYSERR_INVALPARAM';
    MMSYSERR_NODRIVER: Result:= 'MMSYSERR_NODRIVER';
    else Result:= 'Unknown mixer error';
  end;
end;

end.

