unit uMixerLog;

interface

procedure SaveMixerToFile(aName: String);

implementation

uses
  MMSystem, Sysutils,
  uCommon;


procedure SaveMixerToFile(aName: String);
var aFile: TextFile;
    aCaps: TMixerCaps;
    aMixerLine: MixerLine;
    aControls: MIXERLINECONTROLS;
    aCtrlsArray: array of TMIXERCONTROL;
    aDetails: TMIXERCONTROLDETAILS;
    iDevice, iDest, iSrc, iCtrl: Integer;
    aError: Integer;
begin
  AssignFile(aFile, aName);
  {$I-}
  Rewrite(aFile);
  {$I+}
  if IOResult <> 0 then
    Exit;
  try
    if mixerGetNumDevs < 1 then begin
      Writeln(aFile, 'No sound device found.');
      Exit;
    end;
    Writeln(aFile, '--- Data mask ---');
    Writeln(aFile, '- Line type ID, "Name": [Target Type] <Component Type> Channels');
    Writeln(aFile, '-  "Control Name" <Control Type>');
    Writeln(aFile,  '-----------------');
    aMixerLine.cbStruct:= SizeOf(aMixerLine);
    aControls.cbStruct:= SizeOf(aControls);
    aControls.cbmxctrl:= SizeOf(TMIXERCONTROL);
    aDetails.cbStruct:= SizeOf(TMIXERCONTROLDETAILS);
    aDetails.cbDetails:= 4;
    for iDevice:= 0 to mixerGetNumDevs - 1 do begin
      aError:= mixerGetDevCaps(iDevice, @aCaps, SizeOf(aCaps));
      if aError<>0 then
        Exit;
      Writeln(aFile, 'Device ', iDevice, ', ', aCaps.szPName);
      for iDest:= 0 to aCaps.cDestinations - 1 do begin
        aMixerLine.dwDestination:= iDest;
        aError:= mixerGetLineInfo(iDevice, @aMixerLine, MIXER_GETLINEINFOF_DESTINATION);
        if aError<>0 then
          Exit;
        Writeln(aFile, ' Destination ', iDest, ', ', LineInfo(@aMixerLine));
        if aMixerLine.cControls > 0 then begin
          SetLength(aCtrlsArray, aMixerLine.cControls);
          aControls.pamxctrl:= @aCtrlsArray[0];
          aControls.dwLineID:= aMixerLine.dwLineID;
          aControls.cControls:= aMixerLine.cControls;
          if 0<>mixerGetLineControls(iDevice, @aControls, MIXER_GETLINECONTROLSF_ALL) then
            Exit;
          for iCtrl:= 0 to aControls.cControls - 1 do
            Writeln(aFile, '   ', CtrlInfo(@aCtrlsArray[iCtrl]));
        end;
        for iSrc:= 0 to aMixerLine.cConnections - 1 do begin
          aMixerLine.dwSource:= iSrc;
          if 0<>mixerGetLineInfo(iDevice, @aMixerLine, MIXER_GETLINEINFOF_SOURCE) then
            Exit;
          Writeln(aFile, '  Source ', iSrc, ', ', LineInfo(@aMixerLine));
          if aMixerLine.cControls > 0 then begin
            SetLength(aCtrlsArray, aMixerLine.cControls);
            aControls.pamxctrl:= @aCtrlsArray[0];
            aControls.dwLineID:= aMixerLine.dwLineID;
            aControls.cControls:= aMixerLine.cControls;
            if 0<>mixerGetLineControls(iDevice, @aControls, MIXER_GETLINECONTROLSF_ALL) then
              Exit;
            for iCtrl:= 0 to aControls.cControls - 1 do
              Writeln(aFile, '   ', CtrlInfo(@aCtrlsArray[iCtrl]));
          end;    
        end;
      end;
    end;
    Writeln(aFile, '------ End ------');
  finally
    CloseFile(aFile);
  end;
end;

end.
